## NicheProgA.R

## Program to calculate niche overlaps at a single site.

## Program reads in individual data,  calculates 
## electivity scores for any resource usage variables,
## calculate niche overlaps between pairs of species, and
## runs null models to find p-values for testing between 
## species (or other taxonomic classification).

## See the file  NicheInfo.pdf  for more details.

## Cut and paste the following commands into R.
## They may be pasted into R in groups of rows - all comments 
## are preceded by a hash, so will not be acted on in R.

## Sections of the program which require input from the user
## are bracketed by

## ?????????

## Cut and paste the following commands into R, but modifying
## the commands bracketed by ????????? to suit your data.


## -----------------------------------------------------------
##             Functions Input
## -----------------------------------------------------------

## ?????????
## Open R, clear workspace:

rm(list=ls())   

## Read in the niche overlap functions:

source("NicheFunctions.R")

## ?????????


## -----------------------------------------------------------
##          Analysis A: Analysis at a single site
## -----------------------------------------------------------

## Type A analysis is for a single site. Pairs of species
## have their niche overlaps calculated and tested for
## significance. Individual-based variables are used, and 
## electivity may be included as a categorical type of data.

## The input data set is a .csv file.
## It needs to have its first two columns
## labelled "id" and "species". Subsequent columns are
## individual-based variables, one per column.

## Input the individual data file, naming it A.df:

## ?????????  
A.df <- read.csv("ExampleA.csv",T)
## ?????????  

str(A.df)   ## Should be a data frame

## Ensure the first two column names are "id" and "species".

names(A.df)
## If necessary:
names(A.df)[1] <- "id"
names(A.df)[2] <- "species"


## Ensure that the first 2 cols are factors.
## If necessary:
A.df$id      <- as.factor(A.df$id)
A.df$species <- as.factor(A.df$species)


## Store some vectors of names:

spnames   <- levels(A.df$species)
no.spp    <- length(spnames)

varnames <- colnames(A.df)[-(1:2)]    
no.vars  <- length(varnames) 

## Make a vector of variable types to match the variable names:

## "cat"   = categorical, but not resource selection
## "bin"   = binary
## "cts"   = continuous, use raw data (no transformation)
## "meas"  = measurement, continuous positive, take logs
## "pcent" = percentage data, bounds at 0 and 100, use logits
## "propn" = proportion data, bounds at 0 and 1, use logits
## "count" = count
## "rsel"  = resource selection, categorical

## The following command goes with the ExampleA data set:

## ?????????
vartypes <- c("rsel","count","cat","bin","propn","meas")
## ?????????

## Check they are correctly labelled:

cbind(varnames,vartypes)

## Any variables of resource selection type should have an
## associated availability vector for the site.

## Set up a list of objects which are NULL if this is not
## a resource selection variable, and with the availability
## vector if it is resource selection.

avail.list <- vector("list",no.vars)
names(avail.list) <- varnames

## Set the right matrices into the list.
## Run through the following routine for each resource selection
## type of variable.

## ?????????
## ----------------------------------------------------------------
## Routine start:
## In the ExampleA data, just the first variable "habitat" is of
## type "rsel" =  resource selection.

## Read in the availability vector for this resource from a file.
## These are percentages of the various choices.

avail.vect <- read.csv("ExampleAhabitatAvail.csv",T)[1,]

## Sort alphabetically:

avail.vect <- avail.vect[sort.list(names(avail.vect))]

## Check that the resource types used are all in the availability
## vector, need names to match.

used <- levels(A.df$habitat)
used %in% names(avail.vect)

## If not all TRUE, go back to data files, rename to make them match,
## start analysis again.

## Ensure the availabilities are percentages:

avail.vect <- avail.vect/sum(avail.vect)*100
## If they were already percentages, this makes no change.

avail.list[[1]] <- avail.vect  ## Stored in first component, as
                               ## "habitat" was the first variable.

## Routine end.
## -----------------------------------------------------------------
## ?????????

## Read in more availability data if required for other variables.
## Do the routine above, between the dashed lines, for each "rsel"
## variable.
## Put them in the correct component of the list.

## Next look at avail.list, check it seems right - availability
## vectors should match "rsel" type variables, NULL elsewhere.
## Different options within each resource should be in alphabetical
## order.

avail.list




## Set up R objects to store results
## ---------------------------------

## alpha.list
## ----------

## The object alpha.list has one component per variable.
## The components are NULL for ordinary variables.
## For resource selection variables, the component is
## the matrix of Manly's alpha values for that variable. 
## The matrix has:
## Rows = species,
## Cols = choices for that resource (e.g. the resource
## "habitat" may have choices "grass", "rock", "forest"),

alpha.list <- vector("list",no.vars)
names(alpha.list) <- varnames

for (vv in 1:no.vars) if (vartypes[vv]=="rsel"){
    choices <- unique(A.df[,vv+2])
    no.ch   <- length(choices)
    alpha.list[[vv]] <- matrix(NA,no.spp,no.ch)
    dimnames(alpha.list[[vv]]) <- list(spnames,choices)
}


## no.array
## --------

## Set up an array of niche overlaps.
## The object no.array is an array of niche overlaps.
## It is a 3-D array, with rows and columns being species 
## (a square symmetric matrix for pairwise niche overlaps), 
## and the layers are the dimensions for the multivariate 
## niche overlap measure (one dimension per variable).
## Rows and columns are species, layers are variables.

ns <- no.spp
snames <- spnames

no.array  <- array(1,c(no.spp,no.spp,no.vars))
dimnames(no.array) <- list(spnames,spnames,varnames)

## Run through each variable in turn, identify its type,
## calculate the appropriate NO matrix and store it in
## the right layer of the no.array.
for (vv in 1:no.vars){
    print(paste("Variable",vv,":",varnames[vv]))
    y <- A.df[,colnames(A.df)==varnames[vv]]
    if (vartypes[vv] == "bin")
        no.array[,,vv] <- no.bin.fn(A.df$species,y)
    if (vartypes[vv] == "cat")
        no.array[,,vv] <- no.cat.fn(A.df$species,y)
    if (vartypes[vv] == "count")
        no.array[,,vv] <- no.count.fn(A.df$species,y)
    if (vartypes[vv] == "cts")
        no.array[,,vv] <- no.cts.fn(A.df$species,y)
    if (vartypes[vv] == "meas")
        no.array[,,vv] <- no.cts.fn(A.df$species,log(y))
    if (vartypes[vv] == "pcent")
        no.array[,,vv] <- no.cts.fn(A.df$species,
                                    log.fn(y/(100 - y)))
    if (vartypes[vv] == "propn")
        no.array[,,vv] <- no.cts.fn(A.df$species,
                                    log.fn(y/(1 - y)))
    if (vartypes[vv] == "rsel"){
        ## Do Manly's alpha calculations, store.
        no.choices <- length(avail.list[[vv]])
        choicenames <- names(avail.list[[vv]])
        avail.vect <- avail.list[[vv]]
        alpha.mat <- alpha.fn(A.df$species,y,avail.vect)
        alpha.list[[vv]] <- alpha.mat         
        ## Do niche overlaps, as proportions in categories:
        no.array[,,vv] <- no.rsel.cat.fn(alpha.mat)
    }
}

## Also calculate overall NO measures, averaged over the
## dimensions.

no.overall.mat <- apply(no.array,c(1,2),mean)

no.overall.mat



## Analysis A  -  Permutation testing.
## -----------------------------------

## Permutation of the species labels would give data 
## satisfying the null model of complete niche overlap, 
## i.e. that none of the variables 
## serves to differentiate species into different niches.

## Hence for each replication, permute the species labels
## and run through all the calculations above.
## Stor NOs in an array with one extra dimension, one
## layer for each replication.
## Then the null distributions are all stored.
## Can use the original availability data, but need a new 
## alpha list each time.

## Choose no. of replications.
## Start low, eg. with 10 reps, to check it is working.
## Then do more reps, e.g. 100 reps for 2 decimal places in p-values.

replic <- 10

## Set up array to store pseudo niche overlaps:

pseudo.no.array  <- array(1,c(no.spp,no.spp,no.vars,replic))
dimnames(pseudo.no.array) <- list(spnames,spnames,varnames,NULL)

## Set a temporary data frame, which will change each time
## through the cycle by having its species column permuted.

temp.df <- A.df

## For each replication, permute the species labels, run the
## niche overlap calculations, and store the results in the
## pseudo NO array.

for (rr in 1:replic){
    ## Permute the species labels in the temporary dataframe:
    temp.df$species <- sample(temp.df$species)
    print(rr)
    for (vv in 1:no.vars){
        ## Read out the column from this variable:
        y <- temp.df[,colnames(A.df)==varnames[vv]]
        y.save <- y
        ## Run through the variable types, do appropriate analyses:
        if (vartypes[vv] == "bin")
            pseudo.no.array[,,vv,rr] <- no.bin.fn(temp.df$species,y)
        if (vartypes[vv] == "cat")
            pseudo.no.array[,,vv,rr] <- no.cat.fn(temp.df$species,y)
        if (vartypes[vv] == "count")
            pseudo.no.array[,,vv,rr] <- no.count.fn(temp.df$species,y)
        if (vartypes[vv] == "cts")
            pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$species,y)
        if (vartypes[vv] == "meas")
            pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$species,log(y))
        if (vartypes[vv] == "pcent")
            pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$species,
                                                  log.fn(y/(100-y)))
        if (vartypes[vv] == "propn")
            pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$species,
                                                  log.fn(y/(1-y)))
        if (vartypes[vv] == "rsel"){
            ## Do Manly's alpha calculations, store.
            no.choices <- length(avail.list[[vv]])
            choicenames <- names(avail.list[[vv]])
            avail.vect <- avail.list[[vv]]
            alpha.mat  <- alpha.fn(temp.df$species,y,avail.vect)
            ## Do niche overlaps, as proportions in categories:
            pseudo.no.array[,,vv,rr] <- no.rsel.cat.fn(alpha.mat)
        }
        if (is.na(sum(pseudo.no.array[,,vv,rr]))){
            print("Missing niche overlap values found")
            print(paste("Variable type:",vartypes[vv]))
            temp.df.save <- temp.df
        }
    }
    print(paste("Rep",rr,"done"))
}

## Check if there are missing values. 

sum(is.na(c(pseudo.no.array)))


## Note: If the routine above is working with replic=10, go back
## and increase the number of replications. Set it going, and go
## and have a cup of coffee.

## Calculate p values for each pair of species 
## separately over all variables.

sep.pvals     <- array(1,c(no.spp,no.spp,no.vars))
dimnames(sep.pvals) <- list(spnames,spnames,varnames)

for (spa in 1:(no.spp-1)){
    for (spb in (spa+1):no.spp){
        for (vv in 1:no.vars){
            pseudo.nos <- pseudo.no.array[spa,spb,vv,]
            data.no    <- no.array[spa,spb,vv]
            sep.pvals[spa,spb,vv] <- mean(pseudo.nos<data.no) 
            length(pseudo.nos[data.no<pseudo.nos])
            sep.pvals[spb,spa,vv] <- sep.pvals[spa,spb,vv] 
        }
    }
}

sep.pvals


## Also find p value for overall NO measure.

overall.pvals <- matrix(1,no.spp,no.spp)
dimnames(overall.pvals) <- list(spnames,spnames)

for (spa in 1:(no.spp-1)){
    for (spb in (spa+1):no.spp){
        temp.mat  <- pseudo.no.array[spa,spb,,]
        pseudo.nos <- apply(temp.mat,2,mean)
        data.no    <- no.overall.mat[spa,spb]
        overall.pvals[spa,spb] <- mean(pseudo.nos<data.no) 
        length(pseudo.nos[data.no<pseudo.nos])
        overall.pvals[spb,spa] <- overall.pvals[spa,spb] 
    }
}

overall.pvals


## Save all results of the analysis:

A.results <- list(
    info = list(variables = cbind(varnames,vartypes),
        perm.reps = replic),
    separate.pvalues = sep.pvals,
    overall.pvalues = overall.pvals,
    data = A.df,
    NOestimates = no.array,
    NOests.overall = no.overall.mat,
    pseudo.no.array = pseudo.no.array)


## To inspect results later, type in
##    names(A.results)
## to decide what to look at. Then type (e.g.)
##    A.results$NOestimates
## to see that component of the list.

## Save the A.results with a more informative names for 
## your own data set.
