# NicheProgB.txt

# Program to read in site and individual data, calculate 
# electivity scores for any resource usage variables,
# calculate niche overlaps of one species between pairs of 
# sites, and run null models to find p-values for testing 
# between sites.

# See the file  NicheInfo.pdf  for more details.

# Cut and paste the following commands into R.
# They may be pasted into R in groups of rows - all comments 
# are preceded by a hash, so will not be acted on in R.

# Sections of the program which require input from the user
# are bracketed by

# ?????????



# -----------------------------------------------------------
#             Functions Input
# -----------------------------------------------------------


# Read in the niche overlap functions:

source("NicheFunctions.R")



# -----------------------------------------------------------
#    Analysis B: Analysis of one species over several sites
# -----------------------------------------------------------

# Type B analysis is for a single species (or taxon) over two
# or more sites. If there are any resource-selection type of
# variables, the analysis can only be done on data where all
# choices of the resource are present at all sites. 

# Niche overlaps are calculated and tested over pairs of sites
# and the full set of sites.

# The input data set is a .csv file.
# It needs to have its first two columns
# labelled "site" and "id". Subsequent columns are
# individual-based variables, one per column.

# Input the individual data file:

# ?????????  
B.df <- read.csv("ExampleB.csv",T)
# ?????????

# Ensure the first two column names are "site" and "id".

colnames(B.df)[1] <- "site"
colnames(B.df)[2] <- "id"


# Ensure that the first 2 cols are factors.

B.df$id   <- as.factor(B.df$id)
B.df$site <- as.factor(B.df$site)


# Store some vectors of names:

sitenames   <- sort(unique(as.character(B.df$site)))
no.sites    <- length(sitenames)

varnames <- colnames(B.df)[-(1:2)]    
no.vars  <- length(varnames) 

# Make a vector of variable types to match the variable names:

# "cat"   = categorical, but not resource selection
# "bin"   = binary
# "cts"   = continuous, use raw data (no transformation)
# "meas"  = measurement, continuous positive, take logs
# "pcent" = percentage data, bounds at 0 and 100, use logits
# "propn" = proportion data, bounds at 0 and 1, use logits
# "count" = count
# "rsel"  = resource selection, categorical

# ?????????
vartypes <- c("rsel","cat","count")
# ?????????

# Check they are correctly labelled:

cbind(varnames,vartypes)

# Any variables of resource selection type should have an
# associated availability matrix over the different sites.
# We need to remove any choices which are not available all
# sites.

# Set up a list of objects which are NULL if this is not
# a resource selection variable, and with the availability
# vector if it is resource selection.

avail.list <- vector("list",no.vars)
names(avail.list) <- varnames

# Set the right matrices into the list.
# Run through the following routine for each resource selection
# type of variable.

# ----------------------------------------------------------------
# Routine start:
# In the ExampleB data, just the first variable "habitat" is of
# type "rsel" =  resource selection.

# Read the availability matrix from a file.

avail.df  <- read.csv("ExampleBhabitatAvail.csv",T)
avail.mat <- as.matrix(avail.df[,-1])
dimnames(avail.mat) <- list(avail.df[,1],colnames(avail.mat))

# Sort columns alphabetically:

avail.mat <- avail.mat[,sort.list(colnames(avail.mat))]

# Check that the resource types used are all in the availability
# vector, need names to match.

used <- levels(B.df$habitat)
used %in% colnames(avail.mat)

# If not all TRUE, go back to data files, rename to make them match,
# start analysis again.

# If any choice is unavailable at one or more sites, remove
# that choice from the data set:

for (ch.no in ncol(avail.mat):1) if (min(avail.mat[,ch.no])==0)
   {
   this.choice <- colnames(avail.mat)[ch.no]
   print(paste("Choice",this.choice,"is unavailable"))
   B.df <- B.df[!(B.df$habitat==this.choice),] 
   B.df$habitat <- as.factor(as.character(B.df$habitat))
   avail.mat <- avail.mat[,-ch.no]  
   }


# Ensure the availabilities are percentages:

for (i in 1:nrow(avail.mat))
   avail.mat[i,] <- avail.mat[i,]/sum(avail.mat[i,])*100

# If they were already percentages, this makes no change.

avail.list[[1]] <- avail.mat   # Stored in first component, as
                               # "habitat" was the first variable.

# Routine end.
# -----------------------------------------------------------------

# Read in more availability data if required for other variables.
# Do the routine above, between the dashed lines, for each "rsel"
# variable.
# Put them in the correct component of the list.
# Look at avail.list, check it seems right - availability
# matrices should match "rsel" type variables, NULL elsewhere.
# Different options within each resource should be in alphabetical
# order.

avail.list




# Set up R objects to store results
# ---------------------------------

# alpha.list

# The object alpha.list has one component per variable.
# The components are NULL for ordinary variables.
# For resource selection variables, the component is
# the matrix of Manly's alpha values for that variable. 
# The matrix has:
# Rows = sites,
# Cols = choices for that resource (e.g. the resource
# "habitat" may have choices "grass", "rock", "forest"),

alpha.list <- vector("list",no.vars)
names(alpha.list) <- varnames

for (vv in 1:no.vars) if (vartypes[vv]=="rsel")
   {
   choices <- unique(B.df[,vv+2])
   no.ch   <- length(choices)
   alpha.list[[vv]] <- matrix(NA,no.sites,no.ch)
   dimnames(alpha.list[[vv]]) <- list(sitenames,choices)
   }

# no.array

# Set up an array of niche overlaps.
# The object no.array is an array of niche overlaps.
# It is a 3-D array, with rows and columns being species 
# (a square symmetric matrix for pairwise niche overlaps), 
# and the layers are the dimensions for the multivariate 
# niche overlap measure (one dimension per variable).
# Rows and columns are sites, layers are variables.

ns <- no.sites
snames <- sitenames

no.array  <- array(1,c(no.sites,no.sites,no.vars))
dimnames(no.array) <- list(sitenames,sitenames,varnames)


# Run through each variable in turn, identify its type,
# calculate the appropriate NO matrix and store it in
# the right layer of the no.array.
for (vv in 1:no.vars)
   {
   y <- B.df[,colnames(B.df)==varnames[vv]]
   if (vartypes[vv] == "bin")
      no.array[,,vv] <- no.bin.fn(B.df$site,y)
   if (vartypes[vv] == "cat")
      no.array[,,vv] <- no.cat.fn(B.df$site,y)
   if (vartypes[vv] == "count")
      no.array[,,vv] <- no.count.fn(B.df$site,y)
   if (vartypes[vv] == "cts")
      no.array[,,vv] <- no.cts.fn(B.df$site,y)
   if (vartypes[vv] == "meas")
      no.array[,,vv] <- no.cts.fn(B.df$site,log.fn(y))
   if (vartypes[vv] == "pcent")
      no.array[,,vv] <- no.cts.fn(B.df$site,
         log.fn(y/(100-y)))
   if (vartypes[vv] == "propn")
      no.array[,,vv] <- no.cts.fn(B.df$site,
         log.fn(y/(1-y)))
   if (vartypes[vv] == "rsel")
      {
      # Do Manly's alpha calculations, store.
      no.choices <- ncol(avail.list[[vv]])
      choicenames <- colnames(avail.list[[vv]])
      avail.mat <- avail.list[[vv]]
      alpha.mat <- alpha2.fn(B.df$site,y,avail.mat)
      alpha.list[[vv]] <- alpha.mat         
      # Do niche overlaps, as proportions in categories:
      no.array[,,vv] <- no.rsel.cat.fn(alpha.mat)
      }
   }

# Also calculate overall NO measures, averaged over the
# dimensions.

no.overall.mat <- apply(no.array,c(1,2),mean)



# Analysis B  -  Permutation testing.
# -----------------------------------

# Permutation of the species labels would give data 
# satisfying the null model of complete niche overlap, 
# i.e. that none of the variables 
# serves to differentiate species into different niches.

# Hence for each replication, permute the species labels
# and run through all the calculations above.
# Stor NOs in an array with one extra dimension, one
# layer for each replication.
# Then the null distributions are all stored.
# Can use the original availability data, but need a new 
# alpha list each time.

# Choose no. of replications:

replic <- 10

# Set up array to store pseudo niche overlaps:

pseudo.no.array  <- array(1,c(no.sites,no.sites,no.vars,replic))
dimnames(pseudo.no.array) <- list(sitenames,sitenames,varnames,NULL)

# Set a temporary data frame, which will change each time
# through the cycle by having its species column permuted.

temp.df <- B.df

# For each replication, permute the species labels, run the
# niche overlap calculations, and store the results in the
# pseudo NO array.

for (rr in 1:replic)
   {
   # Permute the species labels in the temporary dataframe:
   temp.df$site <- sample(temp.df$site)
   for (vv in 1:no.vars)
      {
      # Read out the column from this variable:
      y <- temp.df[,colnames(B.df)==varnames[vv]]
      # Run through the variable types, do appropriate analyses:
      if (vartypes[vv] == "bin")
         pseudo.no.array[,,vv,rr] <- no.bin.fn(temp.df$site,y)
      if (vartypes[vv] == "cat")
         pseudo.no.array[,,vv,rr] <- no.cat.fn(temp.df$site,y)
      if (vartypes[vv] == "count")
         pseudo.no.array[,,vv,rr] <- no.count.fn(temp.df$site,y)
      if (vartypes[vv] == "cts")
         pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$site,y)
      if (vartypes[vv] == "meas")
         pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$site,log.fn(y))
      if (vartypes[vv] == "pcent")
         pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$site,
            log.fn(y/(100-y)))
      if (vartypes[vv] == "propn")
         pseudo.no.array[,,vv,rr] <- no.cts.fn(temp.df$site,
            log.fn(y/(1-y)))
      if (vartypes[vv] == "rsel")
         {
         # Do Manly's alpha calculations, store.
         no.choices <- ncol(avail.list[[vv]])
         choicenames <- colnames(avail.list[[vv]])
         avail.mat <- avail.list[[vv]]
         alpha.mat  <- alpha2.fn(temp.df$site,y,avail.mat)
         # Do niche overlaps, as proportions in categories:
         pseudo.no.array[,,vv,rr] <- no.rsel.cat.fn(alpha.mat)
         }
      }
   print(paste("Rep",rr,"done"))
   }

# Note: If the routine above is working with replic=10, go back
# and increase the number of replications. Set it going, and go
# and have a cup of coffee.

# Calculate p values for each pair of sites 
# separately over all variables.

sep.pvals     <- array(1,c(no.sites,no.sites,no.vars))
dimnames(sep.pvals) <- list(sitenames,sitenames,varnames)

for (sitea in 1:(no.sites-1)) for (siteb in (sitea+1):no.sites)
   for (vv in 1:no.vars)   
   {
   pseudo.nos <- pseudo.no.array[sitea,siteb,vv,]
   data.no    <- no.array[sitea,siteb,vv]
   sep.pvals[sitea,siteb,vv] <- mean(pseudo.nos<data.no) 
      length(pseudo.nos[data.no<pseudo.nos])
   sep.pvals[siteb,sitea,vv] <- sep.pvals[sitea,siteb,vv] 
   }


# Also find p value for overall NO measure.

overall.pvals <- matrix(1,no.sites,no.sites)
dimnames(overall.pvals) <- list(sitenames,sitenames)

for (sitea in 1:(no.sites-1)) for (siteb in (sitea+1):no.sites)
   {
   temp.mat  <- pseudo.no.array[sitea,siteb,,]
   pseudo.nos <- apply(temp.mat,2,mean)
   data.no    <- no.overall.mat[sitea,siteb]
   overall.pvals[sitea,siteb] <- mean(pseudo.nos<data.no) 
      length(pseudo.nos[data.no<pseudo.nos])
   overall.pvals[siteb,sitea] <- overall.pvals[sitea,siteb] 
   }


# Save all results of the analysis:


B.results <- list(
info = list(variables = cbind(varnames,vartypes),
            perm.reps = replic),
separate.pvalues = sep.pvals,
overall.pvalues = overall.pvals,
data = B.df,
NOestimates = no.array,
NOests.overall = no.overall.mat,
pseudo.no.array = pseudo.no.array)


# To inspect results later, type in
#    names(B.results)
# to decide what to look at. Then type (e.g.)
#    B.results$NOestimates
# to see that component of the list.


# ------------------------------------------------------
